package org.processmining.activityclusterarray.models;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.annotations.AuthoredType;
import org.processmining.framework.annotations.Icon;
import org.processmining.framework.util.HTMLToString;

@AuthoredType(typeName = "Activity cluster array", affiliation = AuthoredType.TUE, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
@Icon(icon = "resourcetype_aca_30x35.png")
public interface ActivityClusterArray extends HTMLToString {

	void init(ActivityClusterArray clusters);
	void init(String label, Set<XEventClass> activities);

	/**
	 * Adds a cluster with the given activities, even if a cluster with these
	 * activities already exists.
	 * 
	 * @param activities
	 *            The given activities.
	 * @return The index of the new cluster.
	 */
	int addCluster(Set<XEventClass> activities);

	/**
	 * Removes the first cluster with the given activities, if present.
	 * 
	 * @param activities
	 *            The given activities.
	 * @return The index of the removed cluster, if present. -1 if such a
	 *         cluster is not present.
	 */
	int removeCluster(Set<XEventClass> activities);

	/**
	 * Adds the given set of activities at the given index.
	 * 
	 * @param index The given index.
	 * @param activities The given set of activities.
	 */
	void addCluster(int index, Set<XEventClass> activities);
	
	/**
	 * Removes the cluster at the given index, if valid.
	 * 
	 * @param index
	 *            The given index.
	 */
	void removeCluster(int index);

	/**
	 * Returns the cluster at the given index, if valid.
	 * 
	 * @param index
	 *            The given index.
	 * @return The cluster at the given index, if valid. null if not valid.
	 */
	Set<XEventClass> getCluster(int index);

	/**
	 * Returns an ArrayList containing all clusters that contain all of the
	 * given activities.
	 * 
	 * @param activities
	 *            The given activities.
	 * @return An ArrayList containing all clusters that contain all of the
	 *         given activities.
	 */
	ArrayList<Set<XEventClass>> getClusters(Set<XEventClass> activities);

	/**
	 * Returns an ArrayList containing all clusters.
	 * 
	 * @return An ArrayList containing all clusters.
	 */
	ArrayList<Set<XEventClass>> getClusters();

	public void setInputs(Set<XEventClass> cluster, Set<XEventClass> activities);
	
	public void setOutputs(Set<XEventClass> cluster, Set<XEventClass> activities);
	
	public Set<XEventClass> getInputs(Set<XEventClass> cluster);
	
	public Set<XEventClass> getOutputs(Set<XEventClass> cluster);
	
	public boolean isConsistent();

	public void importFromStream(InputStream input) throws IOException;
	public void exportToFile(File file) throws IOException;
}
